clear all
close all
clc

addpath('algorithms')
addpath('helpers')

idx_sample = [48,48,5,5,5,5,5];
idx_frame = [0,21,1,45,120,137,153];

path = ['results/'];
mkdir([path, 'DPCP']);
mkdir([path, 'RPCA']);
mkdir([path, 'REAPER']);
type = '_data.mat';
counter = 1;

%% Compute the normal through optimization based subspace methods
for frame = idx_frame
    disp(['processing the ',num2str(frame),'th sample']);
    roadID = sprintf('%03d_%04d',idx_sample(counter),frame);
    roadPath = ['data/', roadID,type];
    load(roadPath);

    X = inliers;
    O = outliers;
    X_tilde = [X O];
    
    %% DPCP-PSGM
    [tDPCP,B_PSGD,~] = DPCP_PSGM_optim(normc(X_tilde), 1,1e-8,100);
    DPCPName = sprintf('%03d_%04d normal',idx_sample(counter),frame);
    DPCPPath = [path, 'DPCP/', DPCPName,type];
    outlier_ratio = size(outliers,2)/size(X_tilde,2);
    save(DPCPPath, 'B_PSGD','tDPCP','outlier_ratio');
    %% REAPER-IRLS
    [tREAPER,distancesREAPER, B_REAPER] = REAPER_IRLS_optim(normc(X_tilde), 1, 1e-12, 1000, 1e-15,tDPCP);
    REAPERCName = sprintf('%03d_%04d normal',idx_sample(counter),frame);
    REAPERPath = [path, 'REAPER/', REAPERCName,type];
    save(REAPERPath, 'B_REAPER','tREAPER');
    %% L21-RPCA
    [tL21, ~,B_L21] = RPCA_L21_OutlierDetection(normc(X_tilde), 1, 2e-3,tDPCP);
    RPCAName = sprintf('%03d_%04d normal',idx_sample(counter),frame);
    RPCAPath = [path, 'RPCA/', RPCAName,type];
    save(RPCAPath, 'B_L21','tL21');

    counter = counter +1;
end

%% Evalute ROC for the normals computed above
Algos = {'DPCP/','REAPER/','RPCA/'};
numAlgos = 3;
ROCrecord = zeros(6,size(idx_frame,2));
% 1.DPCP_PSGM
% 2.REAPER
% 3.L21-RPCA
% 4.RANSAC
% 5.10xRANSAC
% 6.100xRANSAC
counter = 1;
for frame = idx_frame
%   Load 3d point clouds
    disp(['processing the ',num2str(frame),'th sample for subspace methods']);
    roadID = sprintf('%03d_%04d',idx_sample(counter),frame);
    roadPath = ['data/', roadID,type];
    load(roadPath);

    X = inliers;
    O = outliers;
    X_tilde = [X O];
%   Generate ground-truth labeling from X & O
    target = [ones(1,size(X,2)),zeros(1,size(O,2))];
    for idxAlgo = 1:numAlgos
    %     load the normals from the Algos
        basesID = sprintf('%03d_%04d normal',idx_sample(counter),frame);
        basesPath = [path, Algos{idxAlgo}, basesID,type];
        load(basesPath);
    end
%   Compute subspace distances as scores for data points
    score_PSGD = abs(B_PSGD'*normc(X_tilde));
    score_REAPER = abs(B_REAPER'*normc(X_tilde));
    score_RPCA = abs(B_L21'*normc(X_tilde));
%   Compute ROC scores
    [prec,tpr,fpr,~] = prec_rec(score_PSGD, target,'plotROC',0 ,'plotPR',0);
    ROCrecord(1,counter) = 1-trapz(fpr,tpr);
    [~,tpr,fpr,~] = prec_rec(score_REAPER, target,'plotROC',0,'plotPR',0);
    ROCrecord(2,counter) = 1-trapz(fpr,tpr);
    [~,tpr,fpr,~] = prec_rec(score_RPCA, target,'plotROC',0,'plotPR',0);
    ROCrecord(3,counter) = 1-trapz(fpr,tpr);
    counter = counter+1;
end

%% Compute the normal and evalute ROC for RANSAC methods (slow)
n1xtrials = 50;
n10xtrials = 50;
n100xtrials = 20;

scale = -3:0.05:0;
threlist = 10.^scale;
tprs = zeros(1,size(scale,2));
fprs = zeros(1,size(scale,2));
counter = 1;
for i = idx_frame
    disp(['processing the ',num2str(i),'th sample for RANSACs']);
%   Load the 3d point clouds
    roadID = sprintf('%03d_%04d',idx_sample(counter),i);
    roadPath = ['data/', roadID,type];
    load(roadPath);

    X = inliers;
    O = outliers;
    X_tilde = normc([X O]);
%   Generate the ground-truth label from X & O
    target = [ones(1,size(X,2)),zeros(1,size(O,2))];
%   Load the time budget from the PSGD
    basesID = sprintf('%03d_%04d normal',idx_sample(counter),i);
    basesPath = [path, 'DPCP/', basesID,type];
    load(basesPath);
%   Start 1xRANSAC
    AUC = 0;
    for q = 1:n1xtrials
        for j = 1:size(threlist,2)
            disp(['processing the ',num2str(i),'th sample 1x']);
            disp([num2str(q),'/',num2str(n1xtrials),' ',num2str(j),'/',num2str(size(threlist,2))]);
        %     run RANSAC with PSGD-budget
            [~, num_iter_RANSAC, idxRANSAC] = RANSAC_PCA(X_tilde, 3, threlist(j), tDPCP);
            score_RANSAC = zeros(1,size(X_tilde,2));
            score_RANSAC(idxRANSAC) = 1;
            [tpr_q,fpr_q] = RANSACtpr(score_RANSAC,target);
            tprs(j) = tpr_q;
            fprs(j) = fpr_q;
            disp(['RANSAC runs ',num2str(num_iter_RANSAC),' iterations']);
        end
        [fprs index] = sort(fprs);
        AUC = AUC + trapz([0,fprs,1],[0,tprs,1]);
    end
    ROCrecord(numAlgos+1,counter) = AUC / n1xtrials;
%   Start 10xRANSAC
    AUC = 0;
    for q = 1:n10xtrials
        for j = 1:size(threlist,2)
            disp(['processing the ',num2str(i),'th sample 10x']);
            disp([num2str(q),'/',num2str(n10xtrials),' ',num2str(j),'/',num2str(size(threlist,2))]);
        %     run RANSAC with PSGD-budget
            [~, ~, idxRANSAC] = RANSAC_PCA(X_tilde, 3, threlist(j), 10*tDPCP);
            score_RANSAC = zeros(1,size(X_tilde,2));
            score_RANSAC(idxRANSAC) = 1;
            [tpr_q,fpr_q] = RANSACtpr(score_RANSAC,target);
            tprs(j) = tpr_q;
            fprs(j) = fpr_q;
        end
        [fprs index] = sort(fprs);
        AUC = AUC + trapz([0,fprs,1],[0,tprs,1]);
    end
    ROCrecord(numAlgos+2,counter) = AUC / n10xtrials;
%   Start 100xRANSAC
    AUC = 0;
    for q = 1:n100xtrials
        for j = 1:size(threlist,2)
            disp(['processing the ',num2str(i),'th sample 100x']);
            disp([num2str(q),'/',num2str(n100xtrials),' ',num2str(j),'/',num2str(size(threlist,2))]);
        %     run RANSAC with PSGD-budget
            [~, ~, idxRANSAC] = RANSAC_PCA(X_tilde, 3, threlist(j), 100*tDPCP);
            score_RANSAC = zeros(1,size(X_tilde,2));
            score_RANSAC(idxRANSAC) = 1;
            [tpr_q,fpr_q] = RANSACtpr(score_RANSAC,target);
            tprs(j) = tpr_q;
            fprs(j) = fpr_q;
        end
        [fprs index] = sort(fprs);
        AUC = AUC + trapz([0,fprs,1],[0,tprs,1]);
    end
    ROCrecord(numAlgos+3,counter) = AUC / n100xtrials;
    counter = counter+1;
end
save(['ROCtable',type],'ROCrecord');
