close all; clear all;
randn('seed',2018);rand('seed',2018)

%% setup
D = 30;  N = 500;
d = 29; c = D - d;
ratio = 1 ./ (1 ./ 0.7 - 1);
M = floor(N * ratio);
X = [normc( randn(d,N) );zeros(D-d,N)];
O = normc(randn(D,M));
Xtilde = [X O];
obj = @(b)norm(Xtilde'*b,1);
% initialization
[bo,~] = eigs(Xtilde*Xtilde',1,'SM');
maxiter = 200;

%% PSGD with piecewise constant stmuize
K_0 = 30; K = 4;
beta = 1/2;
mu_ls = 0.01; alpha = 0.001; beta_ls = 0.5;
b = bo;
angle_piecewise(1) = asin(norm(b(1:d)));
time = tic;time_piecewise(1) = 0;
%%% line search to determine initial step size
grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
grad_norm = norm(grad)^2;
mu = mu_ls;
obj_old = obj(b);
while obj( normc(b - mu*grad) )> obj_old - alpha*mu*grad_norm
    mu = mu*beta_ls;
end
mu_o = mu;
%%%
for i = 2:maxiter
    grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
    mu = mu_o*beta^(max(0,floor((i-K_0)/K)+1));
    b = normc(b - mu*grad);
    angle_piecewise(i) = asin(norm(b(1:d)));
    time_piecewise(i) = toc(time);
end

%% PSGD with modified line search
mu_ls = 0.01; alpha = 0.001; beta_ls = 0.5;
b = bo;
angle_line(1) = asin(norm(b(1:d)));
time = tic;time_line(1) = 0;
obj_old = obj(b);
mu_threshold = 1e-15;  % here mu_threshold is used to reduce the total time,
%the smaller them are, closer b to the complematry subspace, but more time needed
mu = mu_ls;
for i = 2:maxiter
    grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
    grad_norm = norm(grad)^2;
    while (obj( normc(b - mu*grad) )> obj_old - alpha*mu*grad_norm)& mu>mu_threshold;
        mu = mu*beta_ls;
    end
    b = normc(b - mu*grad);
    obj_old = obj(b);
    angle_line(i) = asin(norm(b(1:d)));
    time_line(i) = toc(time);
end

%% IRLS
% DPCP_IRLS parameters
muilon_J = 10^(-12);
delta = 10^(-14);
w = ones(N+M,1);
B = bo;
angle_IRLS(1) = asin(norm(B(1:d)))*90*2/pi;
time = tic;time_IRLS(1) = 0;
for i = 2:maxiter
    R_X = Xtilde * diag(w) * Xtilde';
    %   [U, S, V] = svd(R_X); B = U(:,end);
    [B,~] = eigs(R_X,c,'SM');
    for j = 1 : N+M
        w(j) = 1/max(delta,norm(B'*Xtilde(:,j)));
    end
    angle_IRLS(i) = asin(norm(B(1:d)));
    time_IRLS(i) = toc(time);
end

%% LP
b = bo;
angle_LP(1) = asin(norm(b(1:d)));
time = tic;time_LP(1) = 0;
for i = 2:15
    b = L1_Gurobi(Xtilde,b);
    b = normc(b);
    angle_LP(i) = asin(norm(b(1:d)));
    time_LP(i) = toc(time);
end

%% plot the results
fontsize = 26;
plotStyle = {'b-','r:','k--','g*-','r:','r--','k-','k:','k--'};
figure
semilogy(0:length(angle_piecewise)-1,angle_piecewise,plotStyle{1},'linewidth',2);
legendInfo{1} = ['PSGM-{PGD}'];
hold on
semilogy(0:length(angle_line)-1,angle_line,plotStyle{2},'linewidth',2);
legendInfo{2} = ['PSGM-{MBLS}'];
semilogy(0:length(angle_IRLS)-1,angle_IRLS,plotStyle{3},'linewidth',2);
legendInfo{3} = ['IRLS'];
T_LP = length(find(angle_LP>0));angle_LP(T_LP+1) = 1e-13;
semilogy(0:length(angle_LP)-1,angle_LP,plotStyle{4},'linewidth',2);
legendInfo{4} = ['ALP'];
ylim([1e-13, pi/2])
xlim([0 maxiter])
legend(legendInfo,'Location','Best')
xlabel('iteration','FontSize',fontsize,'FontName','Times New Roman','Interpreter','LaTex');
ylabel('$\theta$','FontSize',fontsize,'FontName','Times New Roman','Interpreter','LaTex','rot',0);
set(gca,'YDir','normal')
set(gca, ...
    'LineWidth' , 2                     , ...
    'FontSize'  , fontsize              , ...
    'FontName'  , 'Times New Roman'         );
set(gcf, 'Color', 'white');

%%%
figure
loglog(time_piecewise,angle_piecewise,plotStyle{1},'linewidth',2);
hold on
loglog(time_line,angle_line,plotStyle{2},'linewidth',2);
loglog(time_IRLS,angle_IRLS,plotStyle{3},'linewidth',2);
loglog(time_LP,angle_LP,plotStyle{4},'linewidth',2);
ylim([1e-13, pi/2])
xlim([0 max(time_LP((angle_LP<90)))])
legend(legendInfo,'Location','Best')
xlabel('time','FontSize',fontsize,'FontName','Times New Roman','Interpreter','LaTex');
ylabel('$\theta$','FontSize',fontsize,'FontName','Times New Roman','Interpreter','LaTex','rot',0);
set(gca,'YDir','normal')
set(gca, ...
    'LineWidth' , 2                     , ...
    'FontSize'  , fontsize              , ...
    'Xtick'    , [1e-3 1e-2 1e-1 1 10],...
    'FontName'  , 'Times New Roman'         );
set(gcf, 'Color', 'white');