close all; clear all;
randn('seed',2018);rand('seed',2018)


%% setup the data
D = 30; %ambient dimension
N = 500; % number of inliers
ratio = 1 ./ (1 ./ 0.7 - 1); % outlier ratio
M = floor(N * ratio); % number of outliers
d = 29; % subspace dimension
X = [normc( randn(d,N) );zeros(D-d,N)];
O = normc(randn(D,M));
Xtilde = [X O];
obj = @(b)norm(Xtilde'*b,1);
% initialization
[bo,~] = eigs(Xtilde*Xtilde',1,'SM');

%% piecewise constant
%%% line search to determine initial step size
maxiter = 2e2;
K_0 = 30; K = 4;
beta = 1/2;
mu_ls = 0.01; alpha = 0.001; beta_ls = 0.5;
b = bo;
grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
grad_norm = norm(grad)^2;
mu = mu_ls;
obj_old = obj(b);
while obj( normc(b - mu*grad) )> obj_old - alpha*mu*grad_norm
    mu = mu*beta_ls;
end
mu_o = mu;
i = 0;
while  i<= maxiter
    i = i+1;
    grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
    grad_norm = norm(grad)^2;
    mu = mu_o*beta^(max(0,floor((i-K_0)/K)+1));
    b = normc(b - mu*grad);
    value_piecewise(i) = obj(b);
    angle_piecewise(i) = asin(norm(b(1:d)));
end

%% line search
mu_threshold = 1e-15;  % here mu_threshold is used to reduce the total time,
%the smaller it is, the closer b to the complematry subspace, but more time needed
b = bo;obj_old = obj(b);
mu = mu_ls;
i = 0;
while mu>mu_threshold & i<= maxiter
    i = i+1;
    grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
    grad_norm = norm(grad)^2;
    while (obj( normc(b - mu*grad) )> obj_old - alpha*mu*grad_norm)& mu>mu_threshold;
        mu = mu*beta_ls;
    end
    b = normc(b - mu*grad);
    value_line(i) = obj(b);obj_old = value_line(i);
    angle_line(i) = asin(norm(b(1:d)));
end

%% constant step size
b = bo;
mu_o = 1e-4;
Niter = 1000;
for i = 1:Niter
    mu = mu_o;
    b = normc(b - mu*( sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2)) );
    %b = normc(b - mu*( sum( repmat(sinh(b'*X/mu)./cosh(b'*X/mu),n,1).*X, 2)) );
    value_constant(i) = norm(Xtilde'*b,1);
    angle_constant(i) = asin(norm(b(1:d)));
end

%% line decay
b = bo; mu_o = 1e-1;
for i = 1:Niter
    mu = mu_o/i;
    b = normc(b - mu*( sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2)) );
    %b = normc(b - mu*( sum( repmat(sinh(b'*X/mu)./cosh(b'*X/mu),n,1).*X, 2)) );
    value_decaylinear(i) = norm(Xtilde'*b,1);
    angle_decaylinear(i) = asin(norm(b(1:d)));
end

%% root k decay
b = bo;
mu_o = 1e-3;
for i = 1:Niter
    mu = mu_o/sqrt(i);
    grad = sum( repmat(sign(b'*Xtilde),D,1).*Xtilde, 2);
    rgrad = grad - (b'*grad)*b;
    b = normc(b - mu*rgrad);
    value_decayroot(i) = norm(Xtilde'*b,1);
    angle_decayroot(i) = asin(norm(b(1:d)));
end

%%
fontsize = 26;
plotStyle = {'b-','k:','m--','r-','g:'};
figure
semilogy(1:length(angle_constant),angle_constant,plotStyle{1},'linewidth',2);
legendInfo{1} = ['$\mu_k = 10^{-4}$']; hold on
semilogy(1:length(angle_decaylinear),angle_decaylinear,plotStyle{2},'linewidth',2);
legendInfo{2} = ['$\mu_k = {10^{-1}}/{k}$'];
semilogy(1:length(angle_decayroot),angle_decayroot,plotStyle{3},'linewidth',2);
legendInfo{3} = ['$\mu_k = {10^{-3}}/{\sqrt{k}}$'];
semilogy(1:length(angle_piecewise),angle_piecewise,plotStyle{4},'linewidth',2);
legendInfo{4} = ['PSGM-PGD'];
semilogy(1:length(angle_line),angle_line,plotStyle{5},'linewidth',2);
legendInfo{5} = ['PSGM-MBLS'];
ylim([1e-13, max(angle_decaylinear)*2])
legend(legendInfo,'Interpreter','LaTex','Location','Best')
xlabel('iteration','FontSize',fontsize,'FontName','Times New Roman');
ylabel('$\theta$','FontSize',fontsize,'FontName','Times New Roman','Interpreter','LaTex','rot',0);
set(gca,'YDir','normal')
set(gca, ...
    'LineWidth' , 2                     , ...
    'FontSize'  , fontsize              , ...
    'FontName'  , 'Times New Roman'         );
set(gcf, 'Color', 'white');
